
import unittest
import urllib2
from mail           import Mail
from turboAPIClient import TurboAPIClient


class TestTurboClientAPI(unittest.TestCase):
    
    
    
    def setUp(self):
        global USERNAME
        global PASSWORD
        global SERVER_URL
        global FROM_MAIL_FIELD
        global TO_MAIL_FIELD
        global INCORRECT_TO_MAIL_FIELD
        global SUBJECT_CONTENT
        global HTML_CONTENT
        global INCORRECT_SERVER_URL
        global INCORRECT_FROM_MAIL_FIELD
        USERNAME                  = "username" #insert TurboSMTP Account "username"
        PASSWORD                  = "password" #insert TurboSMTP Account "password"
        SERVER_URL                = "https://api.turbo-smtp.com/api/mail/send"
        INCORRECT_SERVER_URL      = "https://mock-url.morg"
        FROM_MAIL_FIELD           = "from@email.com"  #insert email "from" value
        INCORRECT_FROM_MAIL_FIELD = "an_incorrect_@email_address_from"
        TO_MAIL_FIELD             = "to@email.com"  #insert email "to" value
        INCORRECT_TO_MAIL_FIELD   = "an_incorrect_@email_address_to"
        SUBJECT_CONTENT           = "Python Test TurboAPI Send(email) : campo SUBJECT"
        HTML_CONTENT              = "Python Test TurboAPI Send(email) : campo HTML_CONTENT"
        
        
    
       
    def test_messageOK(self):
        mail = Mail()
        mail.setFrom(FROM_MAIL_FIELD)
        mail.setTo(TO_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT)
        
        #self.assertEqual('{"message":"OK"}', TurboAPIClient().sendMail(USERNAME, PASSWORD, SERVER_URL, mail), 'We expect: {"message":"OK"}') 
        returnedMessage = TurboAPIClient().sendMail(USERNAME, PASSWORD, SERVER_URL, mail)
        if '{"message":"OK"}' == returnedMessage:
            self.assertTrue(True)
        else:
            if '{"message":"error","errors":{"message":"error","errors":"nocredit"}}'  == returnedMessage:
                self.assertTrue(True)
            else:
                self.assertTrue(False, "Unknown returned message!!") 
                
                       
    #case: "from" parameter is missing  -->The mail is not send
    def test_message_From_Param_Not_Found(self):
        mail = Mail()
        mail.setTo(TO_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT) 
     
        self.assertEqual('{"message":"error","errors":["missing or not valid sender email (from)"]}', TurboAPIClient().sendMail(USERNAME, PASSWORD, SERVER_URL, mail), 'We expect {"message":"error","errors":["missing or not valid sender email (from)"]}') 
    
    #case: "from" parameter is incorrect  -->The mail is not send
    def test_message_From_Param_Incorrect(self):
        mail = Mail()
        mail.setFrom(INCORRECT_TO_MAIL_FIELD)
        mail.setTo(TO_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT) 
     
        self.assertEqual('{"message":"error","errors":["missing or not valid sender email (from)"]}', TurboAPIClient().sendMail(USERNAME, PASSWORD, SERVER_URL, mail), 'We expect {"message":"error","errors":["missing or not valid sender email (from)"]}') 
    
    #case: "to" parameter is missing  -->The mail is not send
    def test_message_To_Param_Not_Found(self):
        mail = Mail()
        mail.setFrom(FROM_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT) 
        
        self.assertEqual('{"message":"error","errors":["missing recipients (to)"]}', TurboAPIClient().sendMail(USERNAME, PASSWORD, SERVER_URL, mail), 'We expect {"message":"error","errors":["missing recipients (to)"]}') 

    #case: "to" parameter is incorrect  -->The mail is not send
    def test_message_To_Param_Incorrect(self):
        mail = Mail()
        mail.setFrom(FROM_MAIL_FIELD)
        mail.setTo(INCORRECT_TO_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT) 
        jsonResponse = TurboAPIClient().sendMail(USERNAME, PASSWORD, SERVER_URL, mail)
        self.assertEqual('{"message":"error","errors":[["\'' + INCORRECT_TO_MAIL_FIELD + '\' \'to\' email not valid"]]}', jsonResponse, "Response: " + jsonResponse) 


    #case: "from" and "to" parameter are missing  -->The mail is not send
    def test_message_From_And_To_Param_Not_Found(self):
        mail = Mail()
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT) 
        
        self.assertEqual('{"message":"error","errors":["missing or not valid sender email (from)","missing recipients (to)"]}', TurboAPIClient().sendMail(USERNAME, PASSWORD, SERVER_URL, mail), 'We expect {"message":"error","errors":["missing or not valid sender email (from)","missing recipients (to)"]}') 

    #case: "from" and "to" parameter are incorrect  -->The mail is not send
    def test_message_From_And_To_Param_Incorrect(self):
        mail = Mail()
        mail.setFrom(INCORRECT_FROM_MAIL_FIELD)
        mail.setTo(INCORRECT_TO_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT) 
        jsonResponse = TurboAPIClient().sendMail(USERNAME, PASSWORD, SERVER_URL, mail)
        self.assertEqual('{"message":"error","errors":["missing or not valid sender email (from)",["\'' + INCORRECT_TO_MAIL_FIELD + '\' \'to\' email not valid"]]}', jsonResponse, "Response: " + jsonResponse) 



    #case: incorrect authentication credentials (username)
    def test_error412_wrong_username(self):
        mail = Mail()
        mail.setFrom(FROM_MAIL_FIELD)
        mail.setTo(TO_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT)
        
        self.assertRaises( urllib2.HTTPError,  lambda mail: TurboAPIClient().sendMail("", PASSWORD, SERVER_URL, mail), (mail) )
        
        
    #case: incorrect authentication credentials(password)
    def test_error412_wrong_password(self):
        mail = Mail()
        mail.setFrom(FROM_MAIL_FIELD)
        mail.setTo(TO_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT)
        
        self.assertRaises( urllib2.HTTPError,  lambda mail: TurboAPIClient().sendMail(USERNAME, "", SERVER_URL, mail), (mail) )
       
    #case: incorrect resource URL or resource not found
    def test_error404(self):
        mail = Mail()
        mail.setFrom(FROM_MAIL_FIELD)
        mail.setTo(TO_MAIL_FIELD)
        mail.setSubject(SUBJECT_CONTENT)
        mail.setHtmlContent(HTML_CONTENT)
        
        self.assertRaises( urllib2.URLError,  lambda mail: TurboAPIClient().sendMail(USERNAME, PASSWORD, INCORRECT_SERVER_URL, mail), (mail) )
        
  
       
       
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()